annealing <-
function(funfcn,x0)
{
#
# ANNEALING Performs General Simulated Annealing,
#   to minimise a function of several variables, specified in 'funfcn'.
#   Code due to Paul Terrill.
#
#  x0 contains the initial parameter estimates
#  T is the current temperature
#  N is the number of iterations at each temperature
#  s is the number of times we accept a new point for a given temperature
#  r defines the annealing schedule
#  p is the number of parameters
#__________________________________________________________
p<-length(x0);
Tmp<-100; N<-5500; r<-0.90;
x<-x0;
Eold<-do.call("funfcn",list(x0));
s<-1;
while(s>0) {
  s<-0;
  for(i in 1:N) {
#    set.seed(round(runif(1,0,1024))) # sets a different seed for
                                     # the random number generator.
    u1<-runif(1);                    # u1 is used to select a particular
                                     # parameter.
    for(i in 1:p) {
      if(u1 < i/p) {                 # sets a new value for one
        x[i]<-4*(runif(1)-0.5);      # of the parameters according
        u1<-2;                       # to the range of x(i). This needs
      }                              # careful consideration for each
    }                                # problem considered. Cf. 'fminbnd'.
    Enew<-do.call("funfcn",list(x));
    u2<-runif(1);
    del<-Enew-Eold;
    crit<-exp(-del/Tmp);
    if(u2<=crit) {                # checks criterion for accepting
      x0<-x;                      # new point. If the point is accepted,
      Eold<-do.call("funfcn",list(x0));     # we set s=s+1.
      s<-s+1;
    }
  }
  Tmp<-r*Tmp;
}
x0;
}
