# Program to fit a smoothed spline to a set of data, including
# the cross-validation method for choosing the
# smoothing parameter alpha. 
# global variables are : Q, R and Y.
# Variables Y and t are respectively the 'y-' and 'x-' values of the data.
# In this case they are set by a separate program.
# Calls CVSEARCH and FMINBND.
# Code due to Paul Terrill.
#________________________________________________________________________
h<-t[2:m]-t[1:(m-1)]          # we start by forming the Q and
                                # R matrices
ih<-1/h 
qa<-matrix(0,m-2,m-2); qb<-qa
qa[row(qa)-col(qa)==1]<-ih[2:(m-2)]
qb[row(qb)-col(qb)==-1]<-ih[2:(m-2)]
Q<-matrix(0,m-2,m-2)+qa+qb+diag(-ih[1:(m-2)]-ih[2:(m-1)])
Q<-rbind(c(ih[1],rep(0,m-3)),Q,c(rep(0,m-3),ih[m-1]));
ra<-matrix(0,m-2,m-2); rb<-ra
ra[row(ra)-col(ra)==1]<-1/6*h[2:(m-2)]
rb[row(rb)-col(rb)==-1]<-1/6*h[2:(m-2)]
R<-matrix(0,m-2,m-2)+ra+rb+diag(1/3*(h[1:(m-2)]+h[2:(m-1)]))

				# We now need to choose an alpha that
				# minimises the cross-validation 
				# criterion, CV(alpha)
alpha<-seq(0.01,5,0.005)
cv<-rep(0,length(alpha))
for (i in 1:length(alpha)) {
  cv[i]<-cvsearch(alpha[i])
}
X11()
plot(alpha,cv,type="lines") 	# this plots the graph of CV(alpha)
cat("Enter lower value for fminbnd and press return: ")
lv<-scan(file="",n=1)
cat("Enter upper value for fminbnd and press return: ")
uv<-scan(file="",n=1)
alpha<-optimize(cvsearch,c(lv,uv))
alpha<-alpha$minimum
				# we use 'fminbnd' to select alpha
gamma<-solve(R+alpha*t(Q)%*%Q,t(Q)%*%Y)
g<-Y-alpha*Q%*%matrix(gamma,ncol=1)
				# next we plot the splines
dev.set(dev.prev())
y1<-1+9*t-t*t+6;
points(t,y1,pch="o")
gamma<-c(0,gamma,0);
for (i in 1:(m-1)) {
  x<-seq(t[i],t[i+1],0.01)
  q1<-rep(1,length(x))
  t1<-x-t[i]*q1
  t2<-t[i+1]*q1-x
  w1<-(t1*g[i+1]+t2*g[i])/h[i]

				# now we have to account for the unusual 
				# indexing used in gamma, and the fact
				# that the second-order derivatives are
				# zero at each end.

  w2<-t1*t2*(gamma[i+1]*(q1+t1/h[i])+gamma[i]*(q1+t2/h[i]))
  w<-w1-w2/6
  lines(x,w)
}
