expinf <-
function(funfcn,x,m)
{
#
# EXPINF Calculates numerically the expected information matrix of a
#   multinomial model. The function, 'funfcn' gives the probability matrix
#   of the model with the rows corresponding to each independent sample
#   from the model, with corresponding multinomial index m. The sum over
#   each row equals unity.
#   x is the vector of parameter values;
#   m is the vector of multinomial indices.
#   Code due to Ted Catchpole, modified by Paul Terrill.
#
delta<-1e-6*sign(x)*x
t<-length(x)
I<-matrix(0,t,t)
Delta<-diag(delta)
P<-do.call("funfcn",list(x))
c<-ncol(P)
m<-matrix(m,ncol=1)%*%matrix(1,1,c)
for (i in 1:t) {
  for (j in 1:t) {
    dr<-Delta[,i]
    ds<-Delta[,j]
    dPr<-(do.call("funfcn",list(x+dr))-do.call("funfcn",list(x-dr)))/(2*dr[i])
    dPs<-(do.call("funfcn",list(x+ds))-do.call("funfcn",list(x-ds)))/(2*ds[i])
    I[i,j]<-sum(m[P>0]*dPr[P>0]*dPs[P>0]/P[P>0])
  }				# Ref :HL, p82, for use of the relational
}				# operator, '>'
(I+t(I))/2
}
