fmax <-
function(funfcn, x0)
{
#
# FMAX  minimises a negative log-likelihood, provided by 'funfcn' and also
#   produces, (i) an estimate of the gradient at the
#   optimum, to check for convergence; (ii) An estimate
#   of the Hessian at the maximum, and its eigen-values; (iii) an estimate 
#   of standard errors and correlations, 
#   using the observed Fisher information - see Equation (4.2);
#   x0 is the starting-value for the simplex method used in 'fminsolve'
#   Calls GRAD, HESSIAN and FMINSEARCH.
#
	xsol <- nlmin(funfcn, x0)
	x <- xsol$x	# x is the maximum likelihood
	g <- grad(funfcn, x)	# estimate
	cat(g, "\n")
	h <- hessian(funfcn, x)
	d <- eigen(h)
	cat(d$values, "\n")
	cov <- solve(h)	# this now approximates the
# dispersion (var/cov) matrix
	k <- sqrt(diag(cov))	# the approximation to the std., errors
	k1 <- diag(k)
	k2 <- solve(k1)
	cor <- k2 %*% cov %*% k2	# this now approximates
# the correlation matrix
	print(cor * lower.tri(cor, T))	# Ref : HL, p 67
	cor <- cor * lower.tri(cor) + k1
	cbind(x, cor)	# gives output in standard format
}
