# Program to maximise the beta-geometric log-likelihood
# while holding the first variable (t) fixed, and then repeating
# this over a grid of t-values in order to obtain a profile
# log-likelihood with respect to the first variable.
# The scalar 't' is global, so that the function 'beopt' can access
# the value at which 't' is fixed. 'n' is the grid size.
# Calls FMINBND, BEOPT.
#____________________________________________________________________
n <- 100
t <- 0
x <- rep(0, n)
y <- rep(0, n)
for (i in 1:n) {
  t <- t + 0.01
  xsol <- optimize(beopt, c(0,1))		# this obtains the value of the
  x[i] <- xsol$minimum			# second parameter,
					# which maximises the
					# log-likelihood when the first
					# parameter is fixed at t.
  y[i] <- beopt(x[i])			# this is the value for the profile
					# log-likelihood.
}
t1 <- seq(0.01, 1, 0.01)                # the grid of t-values for plotting
X11() 
plot(y, t1, type="l")                   # this is the profile log-likelihood
X11()
plot(x, t1, type="l");                  # this is the path traced in the
                                        # parameter space, as we move along 
                                        # the profile-log-likelihood
