# Program to carry out 'nit' iterations of an EM approach to
# likelihood maximisation, for a simple 
# model for the survival of wild animals. The model is
# fitted to just a single cohort of
# data, of size 'n', resulting in  a 4-year study.
# There are two annual survival probabilities, 'phi_1' and 'phi_2'.
# The remaining probability is 'lam', which is the probability that
# a ring from a dead bird is found and reported.
# The data are generated externally, and in the global vector, 'data'.
# (see for example, Exercise 6.16).We estimate the numbers dead each year,
# by the vector 'd'. The number of
# birds still alive at the end of the study is estimated by the scalar,
# 'a' 
#__________________________________________________________________________
	sr <- sum(data)
	nit <- 50
	phi1 <- 0.422
	phi2 <- 0.763
	lam <- 0.192	# initial estimates of the
# parameters
	n <- 1000	# this is the cohort size
	m <- n - sr	# the total unaccounted for at
# the end of the study
	d <- rep(0, 4)
	d[1] <- n * (1 - phi1) * (1 - lam)	# estimate 'd'
	d[2] <- (d[1] * phi1 * (1 - phi2))/(1 - phi1)
	d[3] <- d[2] * phi2
	d[4] <- d[3] * phi2
	a <- n * (phi1 * phi2^3)	# estimate 'a'
	for(i in 1:nit) {
# now we form the
# maximum-likelihood estimates.
		sd <- sum(d)	# This is the M-step.
		lam <- sr/(sr + sd)
		phi1 <- (sr - data[1] + sd - d[1] + a)/(a + sr + sd)
		phi2 <- (3 * a + d[3] + 2 * d[4] + data[3] + 2 * data[4])/(2 * 
			d[3] + 3 * d[4] + 3 * a + data[2] + 2 * data[3] + 3 * 
			data[4] + d[2])	# Now we carry out the E-stage
		kap <- 1 - lam * (1 - phi1 * phi2^3)
		d[1] <- (m * (1 - phi1) * (1 - lam))/kap
		d[2] <- (d[1] * phi1 * (1 - phi2))/(1 - phi1)
		d[3] <- d[2] * phi2
		d[4] <- d[3] * phi2
		a <- (m * phi1 * (phi2^3))/kap
		cat("\ndata:", data, "\n")
		cat("\nd:", d, "\n")
		cat("\na:", a, "\n")
		cat("\nlam:", lam, "\n")
		cat("\nphi1:", phi1, "\n")
		cat("\nphi2:", phi2, "\n")
}
